@@ -22,17 +22,20 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
+/* Handle the multiplexed channels, such as sessions, x11, agent connections */
+
 #include "includes.h"
 #include "session.h"
 #include "packet.h"
 #include "ssh.h"
 #include "buffer.h"
-#include "util.h"
+#include "dbutil.h"
 #include "channel.h"
 #include "chansession.h"
 #include "ssh.h"
 #include "x11fwd.h"
 #include "agentfwd.h"
+#include "localtcpfwd.h"
 
 static void send_msg_channel_open_failure(unsigned int remotechan, int reason,
 		const unsigned char *text, const unsigned char *lang);
@@ -46,36 +49,48 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 		unsigned int exttype);
 static void send_msg_channel_eof(struct Channel *channel);
 static void send_msg_channel_close(struct Channel *channel);
-static void closechannel(struct Channel *channel);
+static void removechannel(struct Channel *channel);
+static void deletechannel(struct Channel *channel);
 static void send_exitsignalstatus(struct Channel *channel);
+static void checkinitdone(struct Channel *channel);
+static void checkclose(struct Channel *channel);
 
-/* initialise channels */
+static void closeinfd(struct Channel * channel);
+static void closeoutfd(struct Channel * channel, int fd);
+static void closechanfd(struct Channel *channel, int fd, int how);
+
+#define FD_UNINIT (-2)
+#define FD_CLOSED (-1)
+
+/* Initialise all the channels */
 void chaninitialise() {
 
 	/* may as well create space for a single channel */
 	ses.channels = (struct Channel**)m_malloc(sizeof(struct Channel*));
 	ses.chansize = 1;
+	ses.chancount = 0;
 	ses.channels[0] = NULL;
 
 	chansessinitialise();
 }
 
+/* Clean up channels, freeing allocated memory */
 void chancleanup() {
 
-	int i;
+	unsigned int i;
 
 	TRACE(("enter chancleanup"));
 	for (i = 0; i < ses.chansize; i++) {
 		if (ses.channels[i] != NULL) {
 			TRACE(("channel %d closing", i));
-			closechannel(ses.channels[i]);
+			removechannel(ses.channels[i]);
 		}
 	}
 	m_free(ses.channels);
 	TRACE(("leave chancleanup"));
 }
 
-/* create a new channel entry, send a reply confirm or failure */
+/* Create a new channel entry, send a reply confirm or failure */
 /* If remotechan, transwindow and transmaxpacket are not know (for a new
  * outgoing connection, with them to be filled on confirmation), they should
  * all be set to 0 */
@@ -83,7 +98,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 		unsigned int transwindow, unsigned int transmaxpacket) {
 
 	struct Channel * newchan;
-	int i, j;
+	unsigned int i, j;
 
 	TRACE(("enter newchannel"));
 	
@@ -117,29 +132,32 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 	newchan = (struct Channel*)m_malloc(sizeof(struct Channel));
 	newchan->type = type;
 	newchan->index = i;
-	newchan->sentclosed = 0;
-	newchan->recveof = newchan->transeof = newchan->erreof = 0;
+	newchan->sentclosed = newchan->recvclosed = 0;
+	newchan->senteof = newchan->recveof = 0;
 
 	newchan->remotechan = remotechan;
 	newchan->transwindow = transwindow;
 	newchan->transmaxpacket = transmaxpacket;
 	
 	newchan->typedata = NULL;
-	newchan->infd = -1;
-	newchan->outfd = -1;
-	newchan->errfd = -1;
+	newchan->infd = FD_UNINIT;
+	newchan->outfd = FD_UNINIT;
+	newchan->errfd = FD_CLOSED; /* this isn't always set to start with */
+	newchan->initconn = 0;
 
 	newchan->writebuf = buf_new(RECV_MAXWINDOW);
 	newchan->recvwindow = RECV_MAXWINDOW;
 	newchan->recvmaxpacket = RECV_MAXPACKET;
 
 	ses.channels[i] = newchan;
+	ses.chancount++;
 
 	TRACE(("leave newchannel"));
 
 	return newchan;
 }
 
+/* Get the channel structure corresponding to a channel number */
 static struct Channel* getchannel(unsigned int chan) {
 	if (chan >= ses.chansize || ses.channels[chan] == NULL) {
 		return NULL;
@@ -147,52 +165,131 @@ static struct Channel* getchannel(unsigned int chan) {
 	return ses.channels[chan];
 }
 
+/* Iterate through the channels, performing IO if available */
 void channelio(fd_set *readfd, fd_set *writefd) {
 
 	struct Channel *channel;
-	int i;
+	unsigned int i;
 
+	/* iterate through all the possible channels */
 	for (i = 0; i < ses.chansize; i++) {
 
 		channel = ses.channels[i];
-		if (channel == NULL || channel->sentclosed) {
+		if (channel == NULL) {
+			/* only process in-use channels */
 			continue;
 		}
 
-		/* read from program/pipe/etc stdout */
-		if (channel->outfd != -1 && channel->transeof == 0 &&
-				FD_ISSET(channel->outfd, readfd)) {
+		/* read from program/pipe stdout */
+		if (channel->outfd >= 0 && FD_ISSET(channel->outfd, readfd)) {
 			send_msg_channel_data(channel, 0, 0);
 		}
-		/* read from program/pipe stderr for interactive sessions */
-		if (channel->errfd != -1 && channel->erreof == 0 &&
-				FD_ISSET(channel->errfd, readfd)) {
+
+		/* read from program/pipe stderr */
+		if (channel->errfd >= 0 && FD_ISSET(channel->errfd, readfd)) {
 				send_msg_channel_data(channel, 1, SSH_EXTENDED_DATA_STDERR);
 		}
-		/* write to program/pipe stdin */
-		if (channel->infd != -1 && channel->recveof == 0 &&
-				FD_ISSET(channel->infd, writefd)) {
-			writechannel(channel);
+
+		/* if we can read from the infd, it might be closed, so we try to
+		 * see if it has errors */
+		if (channel->infd >= 0 && channel->infd != channel->outfd
+				&& FD_ISSET(channel->infd, readfd)) {
+			int ret;
+			ret = write(channel->infd, NULL, 0);
+			if (ret < 0 && errno != EINTR && errno != EAGAIN) {
+				closeinfd(channel);
+			}
 		}
-		
+
+		/* write to program/pipe stdin */
+		if (channel->infd >= 0 && FD_ISSET(channel->infd, writefd)) {
+			if (channel->initconn) {
+				checkinitdone(channel);
+				continue; /* Important not to use the channel after
+							 checkinitdone(), as it may be NULL */
+			} else {
+				writechannel(channel);
+			}
+		}
+	
 		/* handle any listening sockets */
 		if (channel->type == CHANNEL_ID_SESSION) {
 			struct ChanSess * chansess = (struct ChanSess*)channel->typedata;
 #ifndef DISABLE_X11FWD
-			if (chansess->x11fd != -1 && FD_ISSET(chansess->x11fd, readfd)) {
+			if (chansess->x11fd >= 0 && FD_ISSET(chansess->x11fd, readfd)) {
 				x11accept(chansess);
 			}
 #endif
 #ifndef DISABLE_AGENTFWD
-			if (chansess->agentfd != -1 && FD_ISSET(chansess->agentfd,readfd)) {
+			if (chansess->agentfd >= 0 && FD_ISSET(chansess->agentfd,readfd)) {
 				agentaccept(chansess);
 			}
 #endif
 		}
-	} /* foreach channel */
 
+		/* now handle any of the channel-closing type stuff */
+		checkclose(channel);
+
+	} /* foreach channel */
 }
 
+
+/* do all the EOF/close type stuff checking for a channel */
+static void checkclose(struct Channel *channel) {
+
+	if (!channel->sentclosed) {
+
+		/* check for exited */
+		if (channel->type == CHANNEL_ID_SESSION) {
+			if (((struct ChanSess*)channel->typedata)->exited) {
+				closeinfd(channel);
+			}
+		}
+
+		if (!channel->senteof
+			&& channel->outfd == FD_CLOSED 
+			&& channel->errfd == FD_CLOSED) {
+			send_msg_channel_eof(channel);
+		}
+
+		if (channel->infd == FD_CLOSED
+				&& channel->outfd == FD_CLOSED
+				&& channel->errfd == FD_CLOSED) {
+			send_msg_channel_close(channel);
+		}
+	}
+
+	if (channel->sentclosed && channel->recvclosed) {
+		removechannel(channel);
+	}
+}
+
+
+/* Check whether a deferred (EINPROGRESS) connect() was successful, and
+ * if so, set up the channel properly. Otherwise, the channel is cleaned up, so
+ * it is important that the channel reference isn't used after a call to this
+ * function */
+static void checkinitdone(struct Channel *channel) {
+
+	int val;
+	socklen_t vallen = sizeof(val);
+
+	TRACE(("enter checkinitdone"));
+
+	if (getsockopt(channel->infd, SOL_SOCKET, SO_ERROR, &val, &vallen)
+			|| val != 0) {
+		close(channel->infd);
+		deletechannel(channel);
+		TRACE(("leave checkinitdone: fail"));
+	} else {
+		channel->outfd = channel->infd;
+		channel->initconn = 0;
+		TRACE(("leave checkinitdone: success"));
+	}
+}
+
+
+/* send the exit status or the signal causing termination for a session */
 static void send_exitsignalstatus(struct Channel *channel) {
 
 	struct ChanSess * chansess;
@@ -208,6 +305,7 @@ static void send_exitsignalstatus(struct Channel *channel) {
 }
 
 
+/* Send the close message and set the channel as closed */
 static void send_msg_channel_close(struct Channel *channel) {
 
 	TRACE(("enter send_msg_channel_close"));
@@ -222,6 +320,7 @@ static void send_msg_channel_close(struct Channel *channel) {
 
 	encrypt_packet();
 
+	channel->senteof = 1;
 	channel->sentclosed = 1;
 	TRACE(("leave send_msg_channel_close"));
 }
@@ -237,13 +336,14 @@ static void send_msg_channel_eof(struct Channel *channel) {
 
 	encrypt_packet();
 
-	/* we already know that trans/eof channels are closed */
-	send_msg_channel_close(channel);
+	channel->senteof = 1;
 
 	TRACE(("leave send_msg_channel_eof"));
 }
 
-/* only called when we know we can write to a channel, writes as much as
+/* Called to write data out to the server side of a channel (eg a shell or a
+ * program.
+ * Only called when we know we can write to a channel, writes as much as
  * possible */
 static void writechannel(struct Channel* channel) {
 
@@ -252,11 +352,6 @@ static void writechannel(struct Channel* channel) {
 
 	TRACE(("enter writechannel"));
 
-	if (channel->recveof || channel->sentclosed) {
-		TRACE(("leave writechannel: already recveof or sentclosed"));
-		return;
-	}
-
 	buf = channel->writebuf;
 	maxlen = buf->len - buf->pos;
 
@@ -265,23 +360,25 @@ static void writechannel(struct Channel* channel) {
 		if (errno != EINTR) {
 
 			/* no more to write */
-			channel->recveof = 1;
-
-			/* if everything's closed then close it all up */
-			if (channel->transeof && 
-					(channel->erreof || channel->errfd == -1)) {
-				send_msg_channel_close(channel);
-			}
+			closeinfd(channel);
 		}
 		TRACE(("leave writechannel: len <= 0"));
 		return;
 	}
 	
-	/* extend the window */
-	/* TODO - this is inefficient */
 	if (len == maxlen) {
 		buf_setpos(buf, 0);
 		buf_setlen(buf, 0);
+
+		if (channel->recveof) {
+			/* we're closing up */
+			closeinfd(channel);
+			return;
+			TRACE(("leave writechannel: recveof set"));
+		}
+
+		/* extend the window if we're at the end*/
+		/* TODO - this is inefficient */
 		send_msg_channel_window_adjust(channel, buf->size
 				- channel->recvwindow);
 		channel->recvwindow = buf->size;
@@ -291,15 +388,17 @@ static void writechannel(struct Channel* channel) {
 	TRACE(("leave writechannel"));
 }
 
+/* Set the file descriptors for the main select in session.c
+ * This avoid channels which don't have any window available, are closed, etc*/
 void setchannelfds(fd_set *readfd, fd_set *writefd) {
 	
-	int i;
+	unsigned int i;
 	struct Channel * channel;
 	
 	for (i = 0; i < ses.chansize; i++) {
 
 		channel = ses.channels[i];
-		if (channel == NULL || channel->sentclosed == 1) {
+		if (channel == NULL) {
 			continue;
 		}
 
@@ -307,33 +406,38 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 		if (channel->transwindow > 0) {
 
 			/* stdout */
-			if (channel->outfd != -1 && channel->transeof == 0) {
+			if (channel->outfd >= 0) {
 				/* there's space to read more from the program */
 				FD_SET(channel->outfd, readfd);
 			}
-			/* stderr for interactive sessions */
-			if (channel->errfd != -1 && channel->erreof == 0) {
+			/* stderr */
+			if (channel->errfd >= 0) {
 					FD_SET(channel->errfd, readfd);
 			}
 		}
+
+		if (channel->infd >= 0 && channel->infd != channel->outfd) {
+			FD_SET(channel->infd, readfd);
+		}
+
 		/* stdin */
-		if (channel->infd != -1 && channel->recveof == 0 &&
-				channel->writebuf->pos < channel->writebuf->len) {
+		if (channel->infd >= 0 &&
+				(channel->writebuf->pos < channel->writebuf->len ||
+				 channel->initconn)) {
 			/* there's space to write more to the program */
 			FD_SET(channel->infd, writefd);
 		}
 
-		/* handle any listening sockets - should get optimised away if
-		 * we don't have x11 or agent fwd */
+		/* handle any listening sockets if we have x11 or agent fwd */
 		if (channel->type == CHANNEL_ID_SESSION) {
 			struct ChanSess * chansess = (struct ChanSess*)channel->typedata;
 #ifndef DISABLE_X11FWD
-			if (chansess->x11fd != -1) {
+			if (chansess->x11fd >= 0) {
 				FD_SET(chansess->x11fd, readfd);
 			}
 #endif
 #ifndef DISABLE_AGENTFWD
-			if (chansess->agentfd != -1) {
+			if (chansess->agentfd >= 0) {
 				FD_SET(chansess->agentfd, readfd);
 			}
 #endif
@@ -342,6 +446,9 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 
 }
 
+/* handle the channel EOF event, by closing the channel filedescriptor. The
+ * channel isn't closed yet, it is left until the incoming (from the program
+ * etc) FD is also EOF */
 void recv_msg_channel_eof() {
 
 	unsigned int chan;
@@ -357,24 +464,15 @@ void recv_msg_channel_eof() {
 	}
 
 	channel->recveof = 1;
-
-	/* we should close the channel */
-	if (channel->type == CHANNEL_ID_X11 || channel->type == CHANNEL_ID_AGENT) {
-		shutdown(channel->infd, 0);
-	} else {
-		close(channel->infd);
-	}
-	channel->infd = -1;
-
-	if (channel->transeof && (channel->erreof || channel->errfd == -1)
-			&& !channel->sentclosed) {
-		send_msg_channel_close(channel);
+	if (channel->writebuf->len == 0) {
+		closeinfd(channel);
 	}
 
 	TRACE(("leave recv_msg_channel_eof"));
 }
 
 
+/* Handle channel closure(), respond in kind and close the channels */
 void recv_msg_channel_close() {
 
 	unsigned int chan;
@@ -391,20 +489,23 @@ void recv_msg_channel_close() {
 		dropbear_exit("Close for unknown channel");
 	}
 
-	if (!channel->sentclosed) {
-		send_msg_channel_close(channel);
+	channel->recveof = 1;
+	channel->recvclosed = 1;
+
+	if (channel->sentclosed) {
+		removechannel(channel);
 	}
 
-	closechannel(channel);
-	
 	TRACE(("leave recv_msg_channel_close"));
 }
 
-static void closechannel(struct Channel * channel) {
+/* Remove a channel entry, this is only executed after both sides have sent
+ * channel close */
+static void removechannel(struct Channel * channel) {
 
-	TRACE(("enter closechannel"));
+	TRACE(("enter removechannel"));
 	TRACE(("channel index is %d", channel->index));
-	
+
 	buf_free(channel->writebuf);
 
 	/* close the FDs in case they haven't been done
@@ -414,14 +515,26 @@ static void closechannel(struct Channel * channel) {
 
 	if (channel->type == CHANNEL_ID_SESSION) {
 		closechansess(channel);
+		close(channel->errfd);
 	}
 
-	ses.channels[channel->index] = NULL;
-	m_free(channel);
+	deletechannel(channel);
 
-	TRACE(("leave closechannel"));
+	TRACE(("leave removechannel"));
 }
 
+/* Remove a channel entry */
+static void deletechannel(struct Channel *channel) {
+
+	ses.channels[channel->index] = NULL;
+	ses.chancount--;
+	m_free(channel);
+	
+}
+
+
+/* Handle channel specific requests, passing off to corresponding handlers
+ * such as chansession or x11fwd */
 void recv_msg_channel_request() {
 
 	unsigned int chan;
@@ -456,7 +569,9 @@ void recv_msg_channel_request() {
 
 }
 
-/* chan is the remote channel, isextended is 0 if it is normal data, 1
+/* Reads data from the server's program/shell/etc, and puts it in a
+ * channel_data packet to send.
+ * chan is the remote channel, isextended is 0 if it is normal data, 1
  * if it is extended data. if it is extended, then the type is in
  * exttype */
 static void send_msg_channel_data(struct Channel *channel, int isextended,
@@ -467,25 +582,16 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 	unsigned int maxlen;
 	int fd;
 
-	TRACE(("enter send_msg_channel_data"));
-	TRACE(("extended = %d type = %d", isextended, exttype));
+/*	TRACE(("enter send_msg_channel_data"));
+	TRACE(("extended = %d type = %d", isextended, exttype));*/
 
 	CHECKCLEARTOWRITE();
 
 	assert(!channel->sentclosed);
 
 	if (isextended) {
-		if (channel->erreof) {
-			TRACE(("leave send_msg_channel_data: erreof already set"));
-			return;
-		}
-		assert(exttype == SSH_EXTENDED_DATA_STDERR);
 		fd = channel->errfd;
 	} else {
-		if (channel->transeof) {
-			TRACE(("leave send_msg_channel_data: transeof already set"));
-			return;
-		}
 		fd = channel->outfd;
 	}
 	assert(fd >= 0);
@@ -493,8 +599,8 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 	maxlen = MIN(channel->transwindow, channel->transmaxpacket);
 	/* -(1+4+4) is SSH_MSG_CHANNEL_DATA, channel number, string length, and 
 	 * exttype if is extended */
-	maxlen = MIN(maxlen, ses.writepayload->size 
-			- 1 - 4 - 4 - (isextended ? 4 : 0));
+	maxlen = MIN(maxlen, 
+			ses.writepayload->size - 1 - 4 - 4 - (isextended ? 4 : 0));
 	if (maxlen == 0) {
 		TRACE(("leave send_msg_channel_data: no window"));
 		return; /* the data will get written later */
@@ -506,21 +612,10 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 	if (len <= 0) {
 		/* on error etc, send eof */
 		if (errno != EINTR) {
-			
-			if (isextended) {
-				channel->erreof = 1;
-			} else {
-				channel->transeof = 1;
-			}
-			
-			if ((channel->erreof || channel->errfd == -1)
-					&& channel->transeof) {
-				send_msg_channel_eof(channel);
-			}
+			closeoutfd(channel, fd);
+			TRACE(("leave send_msg_channel_data: read err"));
 		}
 		buf_free(buf);
-		TRACE(("leave send_msg_channel_data: len <= 0, erreof %d transeof %d",
-					channel->erreof, channel->transeof));
 		return;
 	}
 	buf_incrlen(buf, len);
@@ -543,7 +638,8 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 }
 
 
-/* when we receive channel data */
+/* when we receive channel data, put it in a buffer for writing to the program/
+ * shell etc */
 void recv_msg_channel_data() {
 
 	unsigned int chan;
@@ -557,11 +653,16 @@ void recv_msg_channel_data() {
 	chan = buf_getint(ses.payload);
 	channel = getchannel(chan);
 	if (channel == NULL) {
-		/* disconnect ? */
 		dropbear_exit("Unknown channel");
 	}
 
-	assert(channel->infd != -1);
+	if (channel->recveof) {
+		dropbear_exit("received data after eof");
+	}
+
+ 	if (channel->infd < 0) {
+		dropbear_exit("received data with bad infd");
+	}
 
 	datalen = buf_getint(ses.payload);
 
@@ -582,6 +683,8 @@ void recv_msg_channel_data() {
 	buf_setpos(channel->writebuf, pos); /* revert pos */
 
 	channel->recvwindow -= datalen;
+
+	/* matt - this might be for later */
 /*	if (channel->recvwindow < RECV_MINWINDOW) {
 		send_msg_channel_window_adjust(channel, 
 				RECV_MAXWINDOW - channel->recvwindow);
@@ -591,6 +694,9 @@ void recv_msg_channel_data() {
 	TRACE(("leave recv_msg_channel_data"));
 }
 
+/* Increment the outgoing data window for a channel - the remote end limits
+ * the amount of data which may be transmitted, this window is decremented
+ * as data is sent, and incremented upon receiving window-adjust messages */
 void recv_msg_channel_window_adjust() {
 
 	unsigned int chan;
@@ -601,7 +707,7 @@ void recv_msg_channel_window_adjust() {
 	channel = getchannel(chan);
 
 	if (channel == NULL) {
-		dropbear_exit("Unknown channel"); /* TODO - disconnect */
+		dropbear_exit("Unknown channel");
 	}
 	
 	incr = buf_getint(ses.payload);
@@ -613,6 +719,8 @@ void recv_msg_channel_window_adjust() {
 
 }
 
+/* Increment the incoming data window for a channel, and let the remote
+ * end know */
 static void send_msg_channel_window_adjust(struct Channel* channel, 
 		unsigned int incr) {
 
@@ -626,6 +734,7 @@ static void send_msg_channel_window_adjust(struct Channel* channel,
 	encrypt_packet();
 }
 	
+/* Handle a new channel request, performing any channel-type-specific setup */
 void recv_msg_channel_open() {
 
 	unsigned char* type;
@@ -653,6 +762,10 @@ void recv_msg_channel_open() {
 	}
 	if (strcmp(type, "session") == 0) {
 		typeval = CHANNEL_ID_SESSION;
+#ifndef DISABLE_LOCALTCPFWD
+	} else if (strcmp(type, "direct-tcpip") == 0) {
+		typeval = CHANNEL_ID_TCPDIRECT;
+#endif
 	} else {
 		goto failure;
 	}
@@ -668,6 +781,13 @@ void recv_msg_channel_open() {
 	/* type specific initialisation */
 	if (typeval == CHANNEL_ID_SESSION) {
 		newchansess(channel);
+#ifndef DISABLE_LOCALTCPFWD
+	} else if (typeval == CHANNEL_ID_TCPDIRECT) {
+		if (newtcpdirect(channel) == DROPBEAR_FAILURE) {
+			deletechannel(channel);
+			goto failure;
+		}
+#endif
 	}
 
 	/* success */
@@ -684,6 +804,7 @@ cleanup:
 	TRACE(("leave recv_msg_channel_open"));
 }
 
+/* Send a failure message */
 void send_msg_channel_failure(struct Channel *channel) {
 
 	TRACE(("enter send_msg_channel_failure"));
@@ -696,6 +817,7 @@ void send_msg_channel_failure(struct Channel *channel) {
 	TRACE(("leave send_msg_channel_failure"));
 }
 
+/* Send a success message */
 void send_msg_channel_success(struct Channel *channel) {
 
 	TRACE(("enter send_msg_channel_success"));
@@ -708,6 +830,8 @@ void send_msg_channel_success(struct Channel *channel) {
 	TRACE(("leave send_msg_channel_success"));
 }
 
+/* Send a channel open failure message, with a corresponding reason
+ * code (usually resource shortage or unknown chan type) */
 static void send_msg_channel_open_failure(unsigned int remotechan, 
 		int reason, const unsigned char *text, const unsigned char *lang) {
 
@@ -724,6 +848,8 @@ static void send_msg_channel_open_failure(unsigned int remotechan,
 	TRACE(("leave send_msg_channel_open_failure"));
 }
 
+/* Confirm a channel open, and let the remote end know what number we've
+ * allocated and the receive parameters */
 static void send_msg_channel_open_confirmation(struct Channel* channel,
 		unsigned int recvwindow, 
 		unsigned int recvmaxpacket) {
@@ -776,7 +902,8 @@ int send_msg_channel_open_init(int fd, const char * typestring) {
 	return DROPBEAR_SUCCESS;
 }
 
-/* channel establishment is only required if we have listeners (for x11 etc)*/
+/* Confirmation that our channel open request (for forwardings) was 
+ * successful*/
 void recv_msg_channel_open_confirmation() {
 
 	unsigned int chan;
@@ -797,6 +924,7 @@ void recv_msg_channel_open_confirmation() {
 	TRACE(("leave recv_msg_channel_open_confirmation"));
 }
 
+/* Notification that our channel open request failed */
 void recv_msg_channel_open_failure() {
 
 	unsigned int chan;
@@ -808,6 +936,54 @@ void recv_msg_channel_open_failure() {
 		dropbear_exit("Unknown channel");
 	}
 
-	closechannel(channel);
+	removechannel(channel);
 }
+
+/* close a stdout/stderr fd */
+static void closeoutfd(struct Channel * channel, int fd) {
+
+	/* don't close it if it is the same as infd,
+	 * unless infd is already set -1 */
+	TRACE(("enter closeoutfd"));
+	closechanfd(channel, fd, 0);
+	TRACE(("leave closeoutfd"));
+}
+
+/* close a stdin fd */
+static void closeinfd(struct Channel * channel) {
+
+	TRACE(("enter closeinfd"));
+	closechanfd(channel, channel->infd, 1);
+	TRACE(("leave closeinfd"));
+}
+
+/* close a fd, how is 0 for stdout/stderr, 1 for stdin */
+static void closechanfd(struct Channel *channel, int fd, int how) {
+
+	int closein = 0, closeout = 0;
+
+	if (channel->type == CHANNEL_ID_X11 || channel->type == CHANNEL_ID_AGENT
+			|| channel->type == CHANNEL_ID_TCPDIRECT) {
+		shutdown(fd, how);
+		if (how == 0) {
+			closeout = 1;
+		} else {
+			closein = 1;
+		}
+	} else {
+		close(fd);
+		closein = closeout = 1;
+	}
+
+	if (closeout && fd == channel->errfd) {
+		channel->errfd = FD_CLOSED;
+	}
+	if (closeout && fd == channel->outfd) {
+		channel->outfd = FD_CLOSED;
+	}
+	if (closein && fd == channel->infd) {
+		channel->infd = FD_CLOSED;
+	}
+}
+
 #endif /* USE_LISTENERS */
