@@ -26,7 +26,7 @@
 #include "packet.h"
 #include "buffer.h"
 #include "session.h"
-#include "util.h"
+#include "dbutil.h"
 #include "channel.h"
 #include "chansession.h"
 #include "sshpty.h"
@@ -103,7 +103,7 @@ static void sesssigchild_handler(int val) {
 	unsigned int i;
 	struct ChanSess * chansess;
 	struct sigaction sa_chld;
-	
+
 	TRACE(("enter sigchld handler"));
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
 
@@ -120,7 +120,9 @@ static void sesssigchild_handler(int val) {
 				}
 				if (WIFSIGNALED(status)) {
 					chansess->exitsignal = WTERMSIG(status);
+#ifndef AIX
 					chansess->exitcore = WCOREDUMP(status);
+#endif
 				} else {
 					/* we use this to determine how pid exited */
 					chansess->exitsignal = -1;
@@ -725,6 +727,7 @@ static void execchild(struct ChanSess *chansess) {
 
 	/* wipe the hostkey */
 	sign_key_free(ses.opts->hostkey);
+	ses.opts->hostkey = NULL;
 
 	/* overwrite the prng state */
 	seedrandom();
