@@ -27,6 +27,8 @@
 
 
 #include "config.h"
+#include "options.h"
+#include "debug.h"
 
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -51,6 +53,8 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <netdb.h>
+#include <ctype.h>
+#include <stdarg.h>
 
 #include <arpa/inet.h>
 
@@ -74,6 +78,10 @@
 #include <netinet/in.h>
 #endif
 
+#ifdef HAVE_NETINET_TCP_H
+#include <netinet/tcp.h>
+#endif
+
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif
@@ -102,6 +110,9 @@
 #include <libgen.h>
 #endif
 
+#include "libtomcrypt/mycrypt_custom.h"
+#include "libtommath/tommath.h"
+
 #ifndef HAVE_UINT16_T
 #ifndef HAVE_U_INT16_T
 typedef unsigned short u_int16_t;
@@ -109,6 +120,10 @@ typedef unsigned short u_int16_t;
 typedef u_int16_t uint16_t;
 #endif /* HAVE_UINT16_T */
 
+#ifndef HAVE_SOCKLEN_T
+typedef int socklen_t;
+#endif
+
 #ifndef LOG_AUTHPRIV
 #define LOG_AUTHPRIV LOG_AUTH
 #endif
