@@ -22,28 +22,14 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/wait.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <signal.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-
-#include "options.h"
-#include "util.h"
+#include "includes.h"
+#include "dbutil.h"
 #include "session.h"
 #include "buffer.h"
 #include "signkey.h"
 #include "runopts.h"
 
-static void listensocket(int *sock, uint16_t port);
+static int listensockets(int *sock, runopts * opts, int *maxfd);
 static void sigchld_handler(int dummy);
 static void sigsegv_handler(int);
 static void sigintterm_handler(int fish);
@@ -54,8 +40,9 @@ int main(int argc, char ** argv) {
 	
 	fd_set fds;
 	struct timeval seltimeout;
-	int i, j, val;
-	int maxsock;
+	unsigned int i, j;
+	int val;
+	int maxsock = -1;
 	struct sockaddr remoteaddr;
 	int remoteaddrlen;
 	int listensocks[MAX_LISTEN_ADDR];
@@ -72,10 +59,16 @@ int main(int argc, char ** argv) {
 	/* get commandline options */
 	opts = getrunopts(argc, argv);
 
+#ifndef DISABLE_SYSLOG
+	if (usingsyslog) {
+		startsyslog();
+	}
+#endif
 
 	/* fork to background, returning (interactive) users to a term */
-	if (opts->forkbg) {
-
+	if (!opts->forkbg) {
+		dropbear_log(LOG_INFO, "Not forking");
+	} else {
 		switch (fork()) {
 			case -1:
 				dropbear_exit("Failed to create background process");
@@ -84,21 +77,12 @@ int main(int argc, char ** argv) {
 			default:
 				exit(0);
 		}
+
 		if (setpgid(0,0) < 0) {
 			dropbear_exit("Failed to set process group");
 		}
-		startsyslog();
-		fprintf(stderr,"Dropbear: Running in background.\n");
-	} else {
-		fprintf(stderr,"Dropbear: Not forking\n");
+		dropbear_log(LOG_INFO, "Running in background");
 	}
-	
-	/* setup the sockets - we're allowing for multiple listening sockets
-	 * so we can do ip6 etc in future */
-	listensocket(&listensocks[0], opts->port);
-	listensockcount = 1;
-	maxsock = listensocks[listensockcount-1];
-
 
 	/* create a PID file so that we can be killed easily */
 	pidfile = fopen(DROPBEAR_PIDFILE, "w");
@@ -109,7 +93,8 @@ int main(int argc, char ** argv) {
 
 	/* set up cleanup handler */
 	if (signal(SIGINT, sigintterm_handler) == SIG_ERR || 
-		signal(SIGTERM, sigintterm_handler) == SIG_ERR) {
+		signal(SIGTERM, sigintterm_handler) == SIG_ERR ||
+		signal(SIGPIPE, SIG_IGN)) {
 		dropbear_exit("signal() error");
 	}
 
@@ -127,13 +112,17 @@ int main(int argc, char ** argv) {
 	for (i = 0; i < MAX_UNAUTH_CLIENTS; i++) {
 		childpipes[i] = -1;
 	}
+	
+	/* Set up the listening sockets */
+	/* XXX XXX ports */
+	listensockcount = listensockets(listensocks, opts, &maxsock);
 
 	/* incoming connection select loop */
 	for(;;) {
 
 		FD_ZERO(&fds);
 		
-		seltimeout.tv_sec = 60; /* TODO find good value */
+		seltimeout.tv_sec = 60;
 		seltimeout.tv_usec = 0;
 		
 		/* listening sockets */
@@ -159,6 +148,7 @@ int main(int argc, char ** argv) {
 			/* timeout reached */
 			continue;
 		}
+
 		if (val < 0) {
 			if (errno == EINTR) {
 				continue;
@@ -166,7 +156,8 @@ int main(int argc, char ** argv) {
 			dropbear_exit("Listening socket error");
 		}
 
-		/* close fds which have been authed or closed */
+		/* close fds which have been authed or closed - auth.c handles
+		 * closing the auth sockets on success */
 		for (i = 0; i < MAX_UNAUTH_CLIENTS; i++) {
 			if (childpipes[i] >= 0 && FD_ISSET(childpipes[i], &fds)) {
 				close(childpipes[i]);
@@ -179,10 +170,9 @@ int main(int argc, char ** argv) {
 			if (!FD_ISSET(listensocks[i], &fds)) 
 				continue;
 
-			/* child connection */
+			/* child connection XXX - ip6 stuff here */
 			remoteaddrlen = sizeof(struct sockaddr_in);
-			childsock = accept(listensocks[i], 
-					&remoteaddr, &remoteaddrlen);
+			childsock = accept(listensocks[i], &remoteaddr, &remoteaddrlen);
 
 			if (childsock < 0) {
 				/* accept failed */
@@ -195,6 +185,7 @@ int main(int argc, char ** argv) {
 					break;
 				}
 			}
+
 			if (j == MAX_UNAUTH_CLIENTS) {
 				/* no free connections */
 				/* TODO - possibly log, though this would be an easy way
@@ -222,12 +213,12 @@ int main(int argc, char ** argv) {
 
 				/* make sure we close sockets */
 				for (i = 0; i < listensockcount; i++) {
-					if (m_close(listensocks[i]) == -1) {
+					if (m_close(listensocks[i]) == DROPBEAR_FAILURE) {
 						dropbear_exit("Couldn't close socket");
 					}
 				}
 
-				if (m_close(childpipe[0]) == -1) {
+				if (m_close(childpipe[0]) == DROPBEAR_FAILURE) {
 					dropbear_exit("Couldn't close socket");
 				}
 				/* start the session */
@@ -238,8 +229,8 @@ int main(int argc, char ** argv) {
 			
 			/* parent */
 			childpipes[j] = childpipe[0];
-			if (m_close(childpipe[1]) == -1
-						|| m_close(childsock) == -1) {
+			if (m_close(childpipe[1]) == DROPBEAR_FAILURE
+					|| m_close(childsock) == DROPBEAR_FAILURE) {
 				dropbear_exit("Couldn't close socket");
 			}
 		}
@@ -269,57 +260,66 @@ static void sigsegv_handler(int fish) {
 	exit(EXIT_FAILURE);
 }
 
-/* catch ctrl-c or kill */
+/* catch ctrl-c or sigterm */
 static void sigintterm_handler(int fish) {
 
 	exitflag = 1;
 }
 
-static void listensocket(int *sock, uint16_t port) {
+/* Set up listening sockets for all the requested ports */
+static int listensockets(int *sock, runopts * opts, int *maxfd) {
 	
 	int listensock; /* listening fd */
 	struct sockaddr_in listen_addr;
 	struct linger linger;
+	unsigned int i;
 	int val;
 
-	/* open a socket */
-	listensock = socket(PF_INET, SOCK_STREAM, 0);
-	if (listensock < 0) {
-		dropbear_exit("Failed to create socket");
+	for (i = 0; i < opts->portcount; i++) {
+
+		/* iterate through all the sockets to listen on */
+		listensock = socket(PF_INET, SOCK_STREAM, 0);
+		if (listensock < 0) {
+			dropbear_exit("Failed to create socket");
+		}
+
+		val = 1;
+		/* set to reuse, quick timeout */
+		setsockopt(listensock, SOL_SOCKET, SO_REUSEADDR,
+				(void*) &val, sizeof(val));
+		linger.l_onoff = 1;
+		linger.l_linger = 5;
+		setsockopt(listensock, SOL_SOCKET, SO_LINGER,
+				(void*)&linger, sizeof(linger));
+
+		/* disable nagle */
+		setsockopt(listensock, IPPROTO_TCP, TCP_NODELAY,
+				(void*)&val, sizeof(val));
+
+		memset((void*)&listen_addr, 0x0, sizeof(listen_addr));
+		listen_addr.sin_family = AF_INET;
+		listen_addr.sin_port = htons(opts->ports[i]);
+		listen_addr.sin_addr.s_addr = htonl(INADDR_ANY);
+		memset(&(listen_addr.sin_zero), '\0', 8);
+
+		if (bind(listensock, (struct sockaddr *)&listen_addr,
+					sizeof(listen_addr)) < 0) {
+			dropbear_exit("Bind failed port %d", opts->ports[i]);
+		}
+
+		/* listen */
+		if (listen(listensock, 20) < 0) { /* TODO set listen count */
+			dropbear_exit("Listen failed");
+		}
+
+		/* nonblock */
+		if (fcntl(listensock, F_SETFL, O_NONBLOCK) < 0) {
+			dropbear_exit("Failed to set non-blocking");
+		}
+
+		sock[i] = listensock;
+		*maxfd = MAX(listensock, *maxfd);
 	}
 
-	/* set to reuse, quick timeout */
-	setsockopt(listensock, SOL_SOCKET, SO_REUSEADDR,
-			(void*) &val, sizeof(val));
-	linger.l_onoff = 1;
-	linger.l_linger = 5;
-	setsockopt(listensock, SOL_SOCKET, SO_LINGER,
-			(void*)&linger, sizeof(linger));
-
-	val = 1;
-	/* should really use getprotbyname, but we'd need to change "tcp" anyway */
-	setsockopt(listensock, 6, TCP_NODELAY, (void*)&val, sizeof(val));
-
-	/* bind all ip4 addresses */
-	listen_addr.sin_family = AF_INET;
-	listen_addr.sin_port = htons(port);
-	listen_addr.sin_addr.s_addr = htonl(INADDR_ANY);
-	memset(&(listen_addr.sin_zero), '\0', 8); /* XXX neccesary? */
-
-	if (bind(listensock, (struct sockaddr *)&listen_addr, 
-				sizeof(struct sockaddr)) < 0) {
-		dropbear_exit("Bind failed");
-	}
-
-	/* listen */
-	if (listen(listensock, 20) < 0) { /* TODO set listen count */
-		dropbear_exit("Listen failed");
-	}
-
-	/* nonblock */
-	if (fcntl(listensock, F_SETFL, O_NONBLOCK) < 0) {
-		dropbear_exit("Failed to set non-blocking");
-	}
-
-	*sock = listensock;
+	return opts->portcount;
 }
