@@ -24,7 +24,7 @@
 
 #include "includes.h"
 #include "session.h"
-#include "util.h"
+#include "dbutil.h"
 #include "packet.h"
 #include "algo.h"
 #include "buffer.h"
@@ -70,18 +70,23 @@ void child_session(int sock, runopts *opts, int childpipe,
 	FD_ZERO(&readfd);
 	FD_ZERO(&writefd);
 
-	/* main loop, select()s for network and other connections */
+	/* main loop, select()s for all sockets in use */
 	for(;;) {
 
-		TRACE(("top of select loop"));
+
+		if (ses.sock == -1 && ses.chancount == 0) {
+			dropbear_close("Exited normally");
+		}
 		timeout.tv_sec = SELECT_TIMEOUT;
 		timeout.tv_usec = 0;
 		FD_ZERO(&writefd);
 		FD_ZERO(&readfd);
 		assert(ses.payload == NULL);
-		FD_SET(ses.sock, &readfd);
-		if (!isempty(&ses.writequeue)) {
-			FD_SET(ses.sock, &writefd);
+		if (ses.sock != -1) {
+			FD_SET(ses.sock, &readfd);
+			if (!isempty(&ses.writequeue)) {
+				FD_SET(ses.sock, &writefd);
+			}
 		}
 
 		/* set up for channels which require reading/writing */
@@ -89,7 +94,6 @@ void child_session(int sock, runopts *opts, int childpipe,
 			setchannelfds(&readfd, &writefd);
 		}
 		val = select(ses.maxfd+1, &readfd, &writefd, NULL, &timeout);
-		TRACE(("select val = %d", val));
 
 		if (exitflag) {
 			dropbear_exit("Terminated by signal");
@@ -103,6 +107,7 @@ void child_session(int sock, runopts *opts, int childpipe,
 			}
 		}
 
+		/* check for auth timeout, rekeying required etc */
 		checktimeouts();
 		
 		if (val == 0) {
@@ -111,22 +116,25 @@ void child_session(int sock, runopts *opts, int childpipe,
 			continue;
 		}
 
+		/* process session socket's incoming/outgoing data */
+		if (ses.sock != -1) {
+			if (FD_ISSET(ses.sock, &writefd) && !isempty(&ses.writequeue)) {
+				write_packet();
+			}
 
-		if (FD_ISSET(ses.sock, &writefd) && !isempty(&ses.writequeue)) {
-			write_packet();
+			if (FD_ISSET(ses.sock, &readfd)) {
+				read_packet();
+			}
+			
+			/* Process the decrypted packet. After this, the read buffer
+			 * will be ready for a new packet */
+			if (ses.payload != NULL) {
+				process_packet();
+			}
 		}
 
-		if (FD_ISSET(ses.sock, &readfd)) {
-			read_packet();
-		}
-
-		/* Process the decrypted packet. After this, the read buffer
-		 * will be ready for a new packet */
-		if (ses.payload != NULL) {
-			process_packet();
-		}
-
-		/* process pipes etc for the channels */
+		/* process pipes etc for the channels, ses.dataallowed == 0
+		 * during rekeying ) */
 		if (ses.dataallowed == 1) {
 			channelio(&readfd, &writefd);
 		}
@@ -136,7 +144,7 @@ void child_session(int sock, runopts *opts, int childpipe,
 
 /* Check all timeouts which are required. Currently these are the time for
  * user authentication, and the automatic rekeying. */
-void checktimeouts() {
+static void checktimeouts() {
 
 	struct timeval tv;
 	long secs;
@@ -161,6 +169,15 @@ void checktimeouts() {
 	}
 }
 
+/* called when the remote side closes the connection */
+void session_remoteclosed() {
+
+//	/* matt */
+//	close(ses.sock);
+	ses.sock = -1;
+
+}
+
 
 /* clean up a session on exit */
 void session_cleanup() {
@@ -169,6 +186,7 @@ void session_cleanup() {
 	
 	/* we can't cleanup if we don't know the session state */
 	if (!sessinitdone) {
+		TRACE(("leave session_cleanup: !sessinitdone"));
 		return;
 	}
 	
